"""This module is for the translation mapping data of the
following document:

Document file: animation_time.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# animation_time interface": "# animation_time インターフェイス",
    ##################################################
    "This page explains the `animation_time` method interface.": "このページでは`animation_time`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `animation_time` interface returns the current animation elapsed time in milliseconds (`Number` type value).": "`animation_time`インターフェイスは現在のアニメーションの経過時間をミリ秒で返却します（`Number`型の値で設定されます）。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The following example sets the x-coordinate animation of the rectangle and the 1-second interval timer to display an animation's current elapsed time to console (please press the F12 key).": "以下のコード例では四角に対してX座標のアニメーションを設定し、1秒ごとにアニメーションの経過時間をコンソールに出力しています（出力内容はF12キーを押してブラウザで確認してください）。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the animation calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    ap.trace("Animation elapsed time:", rectangle.animation_time())\n\n\nap.Stage(\n    stage_width=500, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.animation_x(x=400, duration=10000).start()\n\noptions: _RectOptions = {"rectangle": rectangle}\nap.Timer(on_timer, delay=1000, options=options).start()\n\nap.save_overall_html(dest_dir_path="animation_time_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The handler that the animation calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    ap.trace("Animation elapsed time:", rectangle.animation_time())\n\n\nap.Stage(\n    stage_width=500, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.animation_x(x=400, duration=10000).start()\n\noptions: _RectOptions = {"rectangle": rectangle}\nap.Timer(on_timer, delay=1000, options=options).start()\n\nap.save_overall_html(dest_dir_path="animation_time_basic_usage/")\n```',  # noqa
    ##################################################
    "## animation_time API": "## animation_time API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get an animation elapsed millisecond.<hr>": "アニメーションの経過時間のミリ秒を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `elapsed_time`: Number": "- `elapsed_time`: Number",
    ##################################################
    "  - An animation elapsed millisecond.": "  - アニメーションの経過時間のミリ秒。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     animation_time: ap.Number = rectangle.animation_time()\n...     ap.trace("animation_time:", animation_time)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(\n...     x=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> ap.Timer(on_timer, delay=ap.FPS.FPS_60, options=options).start()\n```': '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     animation_time: ap.Number = rectangle.animation_time()\n...     ap.trace("animation_time:", animation_time)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_x(\n...     x=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> ap.Timer(on_timer, delay=ap.FPS.FPS_60, options=options).start()\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: animation_base_target.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# AnimationBase class target property interface": "# AnimationBase クラス target 属性のインターフェイス",  # noqa
    ##################################################
    "This page explains the `AnimationBase` class `target` property interface.": "このページでは`AnimationBase`クラスの`target`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What property is this?": "## 属性の概要",
    ##################################################
    "The `target` property returns the animation target instance (e.g., `Sprite`\\, `Rectangle`).": "`target`属性はアニメーション対象のインスタンス（例: `Sprite`や`Rectangle`などのインスタンス）を返却します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "Each subclass of the `AnimationBase` (e.g., `AnimationMove`\\, `AnimationX`) has the `target` getter property.": "`AnimationBase`クラスの各サブクラス（例: `AnimationMove`や`AnimationX`クラスなど）はgetterの`target`属性を持っています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=200, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#00aaff")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100)\nassert isinstance(animation_x.target, ap.Rectangle)\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=200, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#00aaff")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100)\nassert isinstance(animation_x.target, ap.Rectangle)\n```',  # noqa
    ##################################################
    "## Generic type annotation setting": "## ジェネリックの型アノテーションについて",
    ##################################################
    "The `AnimationBase` class and its subclasses can set a generic type annotation. For example, the `target` property type becomes its type if you set it.": "`AnimationBase`クラスとその各サブクラスにはジェネリックの型アノテーションを行うことができます。型アノテーションをした場合`target`属性の型はその型のインスタンスとなります。",  # noqa
    ##################################################
    "The following code sets the `[ap.Rectangle]` generic type annotation:": "以下のコードでは`[ap.Rectangle]`というジェネリックの型アノテーションを行っています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=200, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#00aaff")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX[ap.Rectangle] = rectangle.animation_x(x=100)\nassert isinstance(animation_x.target, ap.Rectangle)\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=200, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#00aaff")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX[ap.Rectangle] = rectangle.animation_x(x=100)\nassert isinstance(animation_x.target, ap.Rectangle)\n```',  # noqa
    ##################################################
    "It is also sometimes useful to annotate generic type to the handler's `AnimationEvent`\\. This generic type annotation also affects the `target` type (`e.this.target`), as follows:": "イベントハンドラの`AnimationEvent`のインスタンスへジェネリックの型アノテーションを行うことも有益なケースがあります。この型アノテーションも`target`属性（`e.this.target`）の型に影響します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    rectangle.animation_x(x=50).start()\n\n\nap.Stage(\n    stage_width=200, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#00aaff")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100)\nanimation_x.animation_complete(on_animation_complete)\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    rectangle.animation_x(x=50).start()\n\n\nap.Stage(\n    stage_width=200, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#00aaff")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100)\nanimation_x.animation_complete(on_animation_complete)\n```',  # noqa
    ##################################################
    "## target property API": "## target 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get an animation target instance.<hr>": "アニメーション対象のインスタンスを取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `target`: VariableNameMixIn": "- `target`: VariableNameMixIn",
    ##################################################
    "  - An animation target instance.": "  - アニメーション対象のインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_animation_complete(\n...     e: ap.AnimationEvent[ap.Rectangle], options: dict\n... ) -> None:\n...     rectangle: ap.Rectangle = e.this.target\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = (\n...     rectangle.animation_x(\n...         x=100,\n...     )\n...     .animation_complete(on_animation_complete)\n...     .start()\n... )\n```': '```py\n>>> import apysc as ap\n>>> def on_animation_complete(\n...     e: ap.AnimationEvent[ap.Rectangle], options: dict\n... ) -> None:\n...     rectangle: ap.Rectangle = e.this.target\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = (\n...     rectangle.animation_x(\n...         x=100,\n...     )\n...     .animation_complete(on_animation_complete)\n...     .start()\n... )\n```',  # noqa
}



"""Common testing helper implementations.
"""

from typing import Any
from typing import Callable
from typing import Dict
from typing import Optional
from typing import Type

import pytest

from apysc._file import file_util


def make_blank_file(*, file_path: str) -> None:
    """
    Make a blank file. If there is no directory, also create
    a parent directory.

    Parameters
    ----------
    file_path : str
        File path to make.
    """
    file_util.save_plain_txt(txt="", file_path=file_path)


def _assert_has_attr(*, any_obj: Any, attr_name: str) -> None:
    """
    Check object has specified attribute.

    Parameters
    ----------
    any_obj : *
        Any object to check.
    attr_name : str
        Expected attribute name.
    """
    msg: str = "Expected attribute does not exists." f"\nAttribute name: {attr_name}"
    assert hasattr(any_obj, attr_name), msg


def assert_attrs(*, expected_attrs: Dict[str, Any], any_obj: Any) -> None:
    """
    Check a specified object's attributes.

    Parameters
    ----------
    expected_attrs : dict
        A dict that has attribute names in key and expected values in value.
    any_obj : *
        Any object to check.

    Raises
    ------
    AssertionError
        If an expected attribute value does not exist or differ.
    """
    for attr_name, expected_value in expected_attrs.items():
        _assert_has_attr(any_obj=any_obj, attr_name=attr_name)

        attr_val: Any = getattr(any_obj, attr_name)
        msg: str = (
            "Attribute value is different from expected value."
            f"\nAttribute name: {attr_name}"
            f"\nAttribute value: {attr_val}"
            f"\nExpected value: {expected_value}"
        )
        assert attr_val == expected_value, msg


def assert_attrs_type(*, expected_types: Dict[str, Type], any_obj: Any) -> None:
    """
    Check a specified object's attribute types.

    Parameters
    ----------
    expected_types : dict
        A dict that has attribute names in key and expected types in value.
    any_obj : *
        Any object to check.

    Raises
    ------
    AssertionError
        If any attribute type differs from an expected type.
    """
    for attr_name, expected_type in expected_types.items():
        _assert_has_attr(any_obj=any_obj, attr_name=attr_name)

        attr_val: Any = getattr(any_obj, attr_name)
        msg: str = (
            "Attribute type is different from expected type."
            f"\nAttribute name: {attr_name}"
            f"\nAttribute type: {type(attr_val)}"
            f"\nExpected type: {expected_type}"
        )
        assert isinstance(attr_val, expected_type), msg


def assert_raises(
    *,
    expected_error_class: type,
    callable_: Callable,
    match: Optional[str] = None,
    **kwargs: Any,
) -> None:
    """
    Check that a specified callable raises an exception.

    Parameters
    ----------
    expected_error_class : type
        Expected error class, for instance, ValueError, Exception, etc.
    callable_ : callable
        Target function or method.
    match : str or None, default None
        Error message's regular expression match pattern.
    **kwargs : dict, optional
        Keyword arguments to pass to the function or method.

    Raises
    ------
    AssertionError
        If a specified interface does not raise an error.
    """
    _kwargs: Dict[str, Any] = {}
    if match is not None:
        _kwargs["match"] = match
    with pytest.raises(expected_error_class, **_kwargs):  # type: ignore
        callable_(**kwargs)



"""This module is for the translation mapping data of the
following document:

Document file: import_conventions.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# import conventions": "# import の慣習",
    ##################################################
    "The apysc library recommends using the `import apysc as ap` to import the package, like the `import numpy as np`\\, `import pandas as pd`\\, `import tkinter as tk`\\, or something like that.": "apyscのライブラリでは他のPythonパッケージの`import numpy as np`、`import pandas as pd`、`import tkinter as tk`などと同じように`import apysc as ap`というimportの指定を推奨しています。",  # noqa
    ##################################################
    "The root package module packages all interfaces you need (for instance, `Sprite`\\, `Int`\\, `Stage`).": "モジュールのルートのパッケージパスにapyscで利用が必要な各インターフェイスが設定されています（例 : `Sprite`、`Int`、`Stage`など）。",  # noqa
    ##################################################
    "Also, it packages internal logic modules with the underscore prefix like the `_file`\\. These packages are not necessary for you.": "また、パッケージの内部で使用するロジックの各モジュールは`_file`のようにアンダースコアのプレフィックスが設定されています。基本的にこれらのプレフィックスの付いたパッケージの利用は不要です。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nnumber_1: ap.Number = int_1 + ap.Number(10.5)\n```": "```py\n# runnable\nimport apysc as ap\n\nint_1: ap.Int = ap.Int(10)\nnumber_1: ap.Number = int_1 + ap.Number(10.5)\n```",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: array_remove_and_remove_at.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Array class remove and remove_at interfaces": "# Array クラスの remove と remove_at のインターフェイス",  # noqa
    ##################################################
    "This page explains the `Array` class `remove` and `remove_at` method interfaces.": "このページでは`Array`クラスの`remove`と`remove_at`メソッドの各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `remove` method removes a specified value from an array, and the `remove_at` method removes a specified index value.": "`remove`メソッドは配列から指定された値を取り除き、`remove_at`メソッドは配列から指定されたインデックスの値を取り除きます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `remove` method requires target value at the first argument, as follows:": "`remove`メソッドは以下のコード例のように取り除く対象の値を第一引数に必要とします。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2, 3])\narr.remove(value=2)\nassert arr == [1, 3]\n```": "```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2, 3])\narr.remove(value=2)\nassert arr == [1, 3]\n```",  # noqa
    ##################################################
    "The `remove_at` method requires index (`int` or `Int` value) at the first argument, as follows:": "`remove_at`メソッドは以下のコード例のように配列のインデックスの整数（Pythonビルトインの`int`もしくはapyscの`Int`）を第一引数に必要とします。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2, 3])\narr.remove_at(index=1)\nassert arr == [1, 3]\n```": "```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2, 3])\narr.remove_at(index=1)\nassert arr == [1, 3]\n```",  # noqa
    ##################################################
    "## remove API": "## remove API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Remove a specified value from this array.<hr>": "指定された値をこの配列から取り除きます。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `value`: Any": "- `value`: Any",
    ##################################################
    "  - Value to remove.": "  - 取り除く対象の値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 3, 5])\n>>> arr.remove(3)\n>>> arr\nArray([1, 5])\n```": "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 3, 5])\n>>> arr.remove(3)\n>>> arr\nArray([1, 5])\n```",  # noqa
    ##################################################
    "## remove_at API": "## remove_at API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Remove a specified index value from this array.<hr>": "指定されたインデックスの値をこの配列から取り除きます。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `index`: Int or int": "- `index`: Int or int",
    ##################################################
    "  - Index to remove value.": "  - 取り除く値のインデックス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.remove_at(1)\n>>> arr\nArray([1, 3])\n```": "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.remove_at(1)\n>>> arr\nArray([1, 3])\n```",  # noqa
}



"""Class implementation for graphic's base class.
"""

from abc import ABC
from abc import abstractmethod
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._display.child_mixin import ChildMixIn
from apysc._display.display_object import DisplayObject
from apysc._display.fill_alpha_mixin import FillAlphaMixIn
from apysc._display.fill_color_mixin import FillColorMixIn
from apysc._display.flip_x_mixin import FlipXMixIn
from apysc._display.flip_y_mixin import FlipYMixIn
from apysc._display.line_alpha_mixin import LineAlphaMixIn
from apysc._display.line_caps import LineCaps
from apysc._display.line_color_mixin import LineColorMixIn
from apysc._display.line_dash_dot_setting import LineDashDotSetting
from apysc._display.line_dash_dot_setting_mixin import LineDashDotSettingMixIn
from apysc._display.line_dash_setting import LineDashSetting
from apysc._display.line_dash_setting_mixin import LineDashSettingMixIn
from apysc._display.line_dot_setting import LineDotSetting
from apysc._display.line_dot_setting_mixin import LineDotSettingMixIn
from apysc._display.line_joints import LineJoints
from apysc._display.line_joints_mixin import LineJointsMixIn
from apysc._display.line_round_dot_setting import LineRoundDotSetting
from apysc._display.line_round_dot_setting_mixin import LineRoundDotSettingMixIn
from apysc._display.rotation_around_center_mixin import RotationAroundCenterMixIn
from apysc._display.rotation_around_point_mixin import RotationAroundPointMixIn
from apysc._display.scale_x_from_center_mixin import ScaleXFromCenterMixIn
from apysc._display.scale_x_from_point_mixin import ScaleXFromPointMixIn
from apysc._display.scale_y_from_center_mixin import ScaleYFromCenterMixIn
from apysc._display.scale_y_from_point_mixin import ScaleYFromPointMixIn
from apysc._display.skew_x_mixin import SkewXMixIn
from apysc._display.skew_y_mixin import SkewYMixIn
from apysc._display.stage import get_stage
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String
from apysc._validation import arg_validation_decos


class GraphicsBase(
    DisplayObject,
    RotationAroundCenterMixIn,
    RotationAroundPointMixIn,
    ScaleXFromCenterMixIn,
    ScaleYFromCenterMixIn,
    ScaleXFromPointMixIn,
    ScaleYFromPointMixIn,
    FlipXMixIn,
    FlipYMixIn,
    SkewXMixIn,
    SkewYMixIn,
    LineColorMixIn,
    LineAlphaMixIn,
    LineJointsMixIn,
    LineDotSettingMixIn,
    LineDashSettingMixIn,
    LineRoundDotSettingMixIn,
    LineDashDotSettingMixIn,
    ABC,
):

    _variable_name: str

    @arg_validation_decos.is_display_object_container(
        arg_position_index=1, optional=True
    )
    @arg_validation_decos.not_empty_string(arg_position_index=2)
    @add_debug_info_setting(module_name=__name__)
    def __init__(self, *, parent: Optional[ChildMixIn], variable_name: str) -> None:
        """
        Vector graphic base class.

        Parameters
        ----------
        parent : ChildMixIn or None
            Parent instance. If a specified value is None,
            this interface uses a stage instance.
        variable_name : str
            Variable name of this instance. This will be used to
            js expression.
        """
        super(GraphicsBase, self).__init__(variable_name=variable_name)
        if parent is None:
            parent = get_stage()
        parent.add_child(child=self)
        self._set_overflow_visible_setting()

    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_basic_values(
        self,
        *,
        fill_color: Union[str, String],
        fill_alpha: Union[float, Number],
        line_color: Union[str, String],
        line_thickness: Union[int, Int],
        line_alpha: Union[float, Number],
        line_cap: Optional[Union[String, LineCaps]],
        line_joints: Optional[Union[String, LineJoints]]
    ) -> None:
        """
        Set initial fundamental values (such as the fill color or
        line thickness).

        Parameters
        ----------
        fill_color : str or String
            A fill-color value to set.
        fill_alpha : float or Number
            A fill-alpha value to set.
        line_color : str or String
            A line-color value to set.
        line_thickness : int or Int
            A line-thickness value to set.
        line_alpha : float or Number
            A line-alpha value to set.
        line_cap : String or LineCaps or None
            A line-cap value to set.
        line_joints : String or LineJoints or None
            A line-joints value to set.
        """
        if isinstance(self, FillColorMixIn):
            self._set_initial_fill_color_if_not_blank(fill_color=fill_color)
        if isinstance(self, FillAlphaMixIn):
            self._update_fill_alpha_and_skip_appending_exp(value=fill_alpha)
        self._set_initial_line_color_if_not_blank(line_color=line_color)
        self._update_line_thickness_and_skip_appending_exp(value=line_thickness)
        self._update_line_alpha_and_skip_appending_exp(value=line_alpha)
        if line_cap is not None:
            self._update_line_cap_and_skip_appending_exp(value=line_cap)
        if line_joints is not None:
            self._update_line_joints_and_skip_appending_exp(value=line_joints)

        if isinstance(self, FillAlphaMixIn):
            self._append_applying_new_attr_val_exp(
                new_attr=self._fill_alpha, attr_name="fill_alpha"
            )
            self._append_attr_to_linking_stack(
                attr=self._fill_alpha, attr_name="fill_alpha"
            )

        if isinstance(self, FillColorMixIn):
            self._append_applying_new_attr_val_exp(
                new_attr=self._fill_color, attr_name="fill_color"
            )
            self._append_attr_to_linking_stack(
                attr=self._fill_color, attr_name="fill_color"
            )

        self._append_applying_new_attr_val_exp(
            new_attr=self._line_color, attr_name="line_color"
        )
        self._append_attr_to_linking_stack(
            attr=self._line_color, attr_name="line_color"
        )

        self._append_applying_new_attr_val_exp(
            new_attr=self._line_alpha, attr_name="line_alpha"
        )
        self._append_attr_to_linking_stack(
            attr=self._line_alpha, attr_name="line_alpha"
        )

        self._append_applying_new_attr_val_exp(
            new_attr=self._line_thickness, attr_name="line_thickness"
        )
        self._append_attr_to_linking_stack(
            attr=self._line_thickness, attr_name="line_thickness"
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_line_setting_if_not_none_value_exists(
        self,
        line_dot_setting: Optional[LineDotSetting],
        line_dash_setting: Optional[LineDashSetting],
        line_round_dot_setting: Optional[LineRoundDotSetting],
        line_dash_dot_setting: Optional[LineDashDotSetting],
    ) -> None:
        """
        If a line setting (dot, dash, or something else) with a value
        other than None exists, set that value to the attribute.

        Parameters
        ----------
        line_dot_setting : Optional[LineDotSetting]
            A dot setting to set.
        line_dash_setting : Optional[LineDashSetting]
            A dash setting to set.
        line_round_dot_setting : Optional[LineRoundDotSetting]
            A round-dot setting to set.
        line_dash_dot_setting : Optional[LineDashDotSetting]
            A dash dot (1-dot chain) setting to set.
        """
        if line_dot_setting is not None:
            self.line_dot_setting = line_dot_setting
            return
        if line_dash_setting is not None:
            self.line_dash_setting = line_dash_setting
            return
        if line_round_dot_setting is not None:
            self.line_round_dot_setting = line_round_dot_setting
            return
        if line_dash_dot_setting is not None:
            self.line_dash_dot_setting = line_dash_dot_setting
            return

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_basic_vals_expression(self, *, expression: str, indent_num: int) -> str:
        """
        Append basic values expression to a specified one.

        Parameters
        ----------
        expression : str
            Target expression.
        indent_num : int
            Indentation number.

        Returns
        -------
        expression : str
            After appending expression.
        """
        from apysc._display import graphics_expression

        if isinstance(self, FillColorMixIn):
            self._initialize_fill_color_if_not_initialized()
            expression = graphics_expression.append_fill_expression(
                fill_color=self._fill_color,
                expression=expression,
                indent_num=indent_num,
            )

        if isinstance(self, FillAlphaMixIn):
            self._initialize_fill_alpha_if_not_initialized()
            expression = graphics_expression.append_fill_opacity_expression(
                fill_alpha=self._fill_alpha,
                expression=expression,
                indent_num=indent_num,
            )

        self._initialize_line_color_if_not_initialized()
        expression = graphics_expression.append_stroke_expression(
            line_color=self._line_color, expression=expression, indent_num=indent_num
        )

        self._initialize_line_thickness_if_not_initialized()
        expression = graphics_expression.append_stroke_width_expression(
            line_thickness=self._line_thickness,
            expression=expression,
            indent_num=indent_num,
        )

        self._initialize_line_alpha_if_not_initialized()
        expression = graphics_expression.append_stroke_opacity_expression(
            line_alpha=self._line_alpha, expression=expression, indent_num=indent_num
        )

        self._initialize_line_cap_if_not_initialized()
        expression = graphics_expression.append_stroke_linecap_expression(
            line_cap=self._line_cap, expression=expression, indent_num=indent_num
        )

        self._initialize_line_joints_if_not_initialized()
        expression = graphics_expression.append_stroke_linejoin_expression(
            line_joints=self._line_joints, expression=expression, indent_num=indent_num
        )

        self._initialize_x_if_not_initialized()
        expression = graphics_expression.append_x_expression(
            x=self._x, expression=expression, indent_num=indent_num
        )

        self._initialize_y_if_not_initialized()
        expression = graphics_expression.append_y_expression(
            y=self._y, expression=expression, indent_num=indent_num
        )
        return expression

    @abstractmethod
    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).
        """



"""Class implementation for 2-dimensional points mix-in.
"""

from typing import Dict
from typing import Tuple

from typing_extensions import final

from apysc._geom.point2d import Point2D
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.array import Array
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_attr_mixin import VariableNameSuffixAttrMixIn
from apysc._validation import arg_validation_decos


class Points2DMixIn(VariableNameSuffixAttrMixIn, VariableNameMixIn, RevertMixIn):

    _points: Array[Point2D]

    @final
    def _initialize_points_if_not_initialized(self) -> None:
        """
        Initialize _points attribute ifa this interface does not
        initialize it yet.
        """
        if hasattr(self, "_points"):
            return
        suffix: str = self._get_attr_variable_name_suffix(attr_identifier="points")
        self._points = Array(
            [],
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

    @property
    @add_debug_info_setting(module_name=__name__)
    def points(self) -> Array[Point2D]:
        """
        Get current points.

        Returns
        -------
        points : Array of Point2D
            Current points.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color="#fff")
        >>> polygon: ap.Polygon = sprite.graphics.draw_polygon(
        ...     points=[
        ...         ap.Point2D(x=0, y=0),
        ...         ap.Point2D(x=50, y=0),
        ...     ]
        ... )
        >>> polygon.points
        Array([Point2D(Int(0), Int(0)), Point2D(Int(50), Int(0))])
        """
        self._initialize_points_if_not_initialized()
        return self._points

    @points.setter
    @arg_validation_decos.is_point_2ds(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def points(self, value: Array[Point2D]) -> None:
        """
        Update points value.

        Parameters
        ----------
        value : Array of Point2D
            Points value to set.

        Raises
        ------
        ValueError
            If array contains not Point2D value.
        """
        self._initialize_points_if_not_initialized()
        self._append_points_update_expression(value=value)
        self._points = value

    @final
    def _make_2dim_points_expression(self) -> Tuple[str, str]:
        """
        Make JavaScript expression string.

        Returns
        -------
        variable_name: str
            Created JavaScript points variable (2-dimensional array) name.
        expression : str
            JavaScript expression string. This expression make
            2-dimensional JavaScript array variable, like
            '[[x_1, y_1], [x_2, y_2], ...]'.
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        self._initialize_points_if_not_initialized()
        variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.ARRAY
        )
        i_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.INDEX
        )
        points_name: str = self._points.variable_name
        point_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.POINT2D
        )
        expression: str = (
            f"var {variable_name} = [];"
            f"\nfor (var {i_name} = 0; {i_name} < {points_name}.length; "
            f"{i_name}++) {{"
            f"\n  var {point_name} = {points_name}[{i_name}];"
            f"\n  {variable_name}.push("
            f'[{point_name}["x"], {point_name}["y"]]);'
            "}"
        )
        return variable_name, expression

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_points_update_expression(self, *, value: Array) -> None:
        """
        Append points updating expression.

        Parameters
        ----------
        value : Array
            Points value to set.
        """
        import apysc as ap

        expression: str = f"{self._points.variable_name} = {value.variable_name};"
        ap.append_js_expression(expression=expression)

    _points_snapshots: Dict[str, Array]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make values' snapshots.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_points_if_not_initialized()
        self._points._run_all_make_snapshot_methods(snapshot_name=snapshot_name)
        self._set_single_snapshot_val_to_dict(
            dict_name="_points_snapshots",
            value=self._points,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert values if snapshots exist.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._points = self._points_snapshots[snapshot_name]
        self._points._run_all_revert_methods(snapshot_name=snapshot_name)



"""Class implementation for the rotation around the given
point animation value.
"""

from typing import Generic
from typing import TypeVar
from typing import Union

from typing_extensions import final

from apysc._animation.animation_base import AnimationBase
from apysc._animation.easing import Easing
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.variable_name_mixin import VariableNameMixIn

_T = TypeVar("_T", bound=VariableNameMixIn)


class AnimationRotationAroundPoint(AnimationBase[_T], Generic[_T]):
    """
    The animation class for a rotation around the given point.

    References
    ----------
    - animation_rotation_around_point interface
        - https://simon-ritchie.github.io/apysc/en/animation_rotation_around_point.html  # noqa
    - Animation interfaces duration setting
        - https://simon-ritchie.github.io/apysc/en/animation_duration.html
    - Animation interfaces delay setting
        - https://simon-ritchie.github.io/apysc/en/animation_delay.html
    - Each animation interface return value
        - https://simon-ritchie.github.io/apysc/en/animation_return_value.html  # noqa
    - Sequential animation setting
        - https://simon-ritchie.github.io/apysc/en/sequential_animation.html
    - animation_parallel interface
        - https://simon-ritchie.github.io/apysc/en/animation_parallel.html
    - Easing enum
        - https://simon-ritchie.github.io/apysc/en/easing_enum.html

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color="#0af")
    >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
    ...     x=50, y=50, width=50, height=50
    ... )
    >>> animation: ap.AnimationRotationAroundPoint
    >>> animation = rectangle.animation_rotation_around_point(
    ...     rotation_around_point=90,
    ...     x=ap.Int(100),
    ...     y=ap.Int(100),
    ...     duration=1500,
    ...     easing=ap.Easing.EASE_OUT_QUINT,
    ... )
    >>> _ = animation.start()
    """

    _rotation_around_point: Int
    _x: Int
    _y: Int
    _before_rotation_around_point: Int
    _rotation_around_point_diff: Int

    @final
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        *,
        target: _T,
        rotation_around_point: Union[int, Int],
        x: Union[int, Int],
        y: Union[int, Int],
        duration: Union[int, Int] = 3000,
        delay: Union[int, Int] = 0,
        easing: Easing = Easing.LINEAR,
    ) -> None:
        """
        The animation class for a rotation around the given point.

        Parameters
        ----------
        target : RotationAroundPointMixIn
            A target instance of the animation target
            (e.g., `Rectangle` instance).
        rotation_around_point : int or Int
            The final rotation around the given point of the animation.
        x : int or Int
            X-coordinate.
        y : int or Int
            Y-coordinate.
        duration : int or Int, default 3000
            Milliseconds before an animation ends.
        delay : int or Int, default 0
            Milliseconds before an animation starts.
        easing : Easing, default Easing.LINEAR
            Easing setting.

        Raises
        ------
        TypeError
            If a specified target is not a RotationAroundPointMixIn
            instance.
        """
        from apysc._converter import to_apysc_val_from_builtin
        from apysc._display.rotation_around_point_mixin import RotationAroundPointMixIn
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.ANIMATION_ROTATION_AROUND_POINT
        )
        self._x = to_apysc_val_from_builtin.get_copied_int_from_builtin_val(integer=x)
        self._y = to_apysc_val_from_builtin.get_copied_int_from_builtin_val(integer=y)
        target_: VariableNameMixIn = target
        if isinstance(target_, RotationAroundPointMixIn):
            target_._initialize_rotation_around_point_if_not_initialized()
            self._before_rotation_around_point = target_.get_rotation_around_point(
                x=self._x, y=self._y
            )
        else:
            raise TypeError(
                "Specified `target` argument is not a "
                f"RotationAroundPointMixIn instance: {type(target_)}"
            )
        self._rotation_around_point = (
            to_apysc_val_from_builtin.get_copied_int_from_builtin_val(
                integer=rotation_around_point
            )
        )
        self._rotation_around_point_diff = (
            self._rotation_around_point - self._before_rotation_around_point
        )
        self._set_basic_animation_settings(
            target=target, duration=duration, delay=delay, easing=easing
        )
        super(AnimationRotationAroundPoint, self).__init__(variable_name=variable_name)

    @final
    def _get_animation_func_expression(self) -> str:
        """
        Get a animation function expression.

        Returns
        -------
        expression : str
            Animation function expression.
        """
        from apysc._type import value_util

        diff_str: str = value_util.get_value_str_for_expression(
            value=self._rotation_around_point_diff
        )
        x_str: str = value_util.get_value_str_for_expression(value=self._x)
        y_str: str = value_util.get_value_str_for_expression(value=self._y)
        return f"\n  .rotate({diff_str}, {x_str}, {y_str});"

    @final
    def _get_complete_event_in_handler_head_expression(self) -> str:
        """
        Get an expression to be inserted into the complete event
        handler's head.

        Returns
        -------
        expression : str
            An expression to insert into the complete event
            handler's head.
        """
        from apysc._display import rotation_interface_helper
        from apysc._display.rotation_around_point_mixin import RotationAroundPointMixIn
        from apysc._type import value_util

        expression: str = ""
        if isinstance(self._target, RotationAroundPointMixIn):
            self._target._initialize_rotation_around_point_if_not_initialized()
            key_exp_str: str = (
                rotation_interface_helper.get_coordinates_key_for_expression(
                    x=self._x, y=self._y
                ).value
            )
            target_rotation_around_point_value_str: str = (
                value_util.get_value_str_for_expression(
                    value=self._target._rotation_around_point
                )
            )
            rotation_around_point_value_str: str = (
                value_util.get_value_str_for_expression(
                    value=self._rotation_around_point
                )
            )
            expression = (
                f"{target_rotation_around_point_value_str}"
                f"[{key_exp_str}] = "
                f"{rotation_around_point_value_str};"
            )
        return expression



"""Class implementation for line color interface.
"""

from typing import Dict
from typing import Union

from typing_extensions import final

from apysc._animation.animation_line_color_mixin import AnimationLineColorMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.string import String
from apysc._type.variable_name_suffix_attr_mixin import VariableNameSuffixAttrMixIn
from apysc._validation import arg_validation_decos


class LineColorMixIn(
    VariableNameSuffixAttrMixIn,
    AnimationLineColorMixIn,
    RevertMixIn,
    AttrLinkingMixIn,
):

    _line_color: String

    @property
    @add_debug_info_setting(module_name=__name__)
    def line_color(self) -> String:
        """
        Get this instance's line color.

        Returns
        -------
        line_color : String
            Current line color (hexadecimal string, e.g., '#00aaff').
            If not be set, this interface returns a blank string.

        References
        ----------
        - GraphicsBase line_color interface
            - https://simon-ritchie.github.io/apysc/en/graphics_base_line_color.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color="#fff", thickness=10)
        >>> line: ap.Line = sprite.graphics.draw_line(
        ...     x_start=50, y_start=50, x_end=150, y_end=50
        ... )
        >>> line.line_color = ap.String("#0af")
        >>> line.line_color
        String('#00aaff')
        """
        import apysc as ap
        from apysc._type import value_util

        self._initialize_line_color_if_not_initialized()
        line_color: ap.String = value_util.get_copy(value=self._line_color)
        return line_color

    @line_color.setter
    @arg_validation_decos.is_hex_color_code_format(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def line_color(self, value: String) -> None:
        """
        Update this instance's line color.

        Parameters
        ----------
        value : str
            Line color to set.

        References
        ----------
        - GraphicsBase line_color interface
            - https://simon-ritchie.github.io/apysc/en/graphics_base_line_color.html  # noqa
        """
        self._initialize_line_color_if_not_initialized()
        self._update_line_color_and_skip_appending_exp(value=value)
        self._append_line_color_update_expression()

        self._append_applying_new_attr_val_exp(
            new_attr=self._line_color, attr_name="line_color"
        )
        self._append_attr_to_linking_stack(
            attr=self._line_color, attr_name="line_color"
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_line_color_update_expression(self) -> None:
        """
        Append line color updating expression.
        """
        import apysc as ap
        from apysc._type import value_util

        line_color_str: str = value_util.get_value_str_for_expression(
            value=self._line_color
        )
        expression: str = f"{self.variable_name}.stroke({line_color_str});"
        ap.append_js_expression(expression=expression)

    @final
    def _set_initial_line_color_if_not_blank(
        self, *, line_color: Union[str, String]
    ) -> None:
        """
        Set initial line color value if a specified value is
        not a blank string.

        Parameters
        ----------
        line_color : str or String
            Line color (hexadecimal string, e.g., '#00aaff').
        """
        import apysc as ap

        self._initialize_line_color_if_not_initialized()
        suffix: str = self._get_attr_variable_name_suffix(attr_identifier="line_color")
        if line_color == "":
            return
        if isinstance(line_color, ap.String):
            line_color_: ap.String = line_color
        else:
            line_color_ = String(line_color, variable_name_suffix=suffix)
        self._update_line_color_and_skip_appending_exp(value=line_color_)

    @final
    def _update_line_color_and_skip_appending_exp(self, *, value: String) -> None:
        """
        Update line color and skip appending expression.

        Parameters
        ----------
        value : String
            Line color to set.
        """
        from apysc._color import color_util

        self._initialize_line_color_if_not_initialized()
        value = color_util.complement_hex_color(hex_color_code=value)
        self._initialize_line_color_if_not_initialized()
        self._line_color = value

    @final
    def _initialize_line_color_if_not_initialized(self) -> None:
        """
        Initialize the line_color attribute if this
        interface does not initialize it yet.
        """
        if hasattr(self, "_line_color"):
            return
        suffix: str = self._get_attr_variable_name_suffix(attr_identifier="line_color")
        self._line_color = String(
            "",
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

    _line_color_snapshots: Dict[str, str]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_line_color_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_line_color_snapshots",
            value=self._line_color._value,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._line_color._value = self._line_color_snapshots[snapshot_name]



"""The module to define keyword link mappings.
"""

from typing import Dict

_Keyword = str
_DocFileName = str

MAPPINGS: Dict[_Keyword, _DocFileName] = {
    "Stage": "stage",
    "Sprite": "sprite",
    "save_overall_html": "save_overall_html",
    "add_child": "add_child_and_remove_child",
    "remove_child": "add_child_and_remove_child",
    "display_on_jupyter": "display_on_jupyter",
    "display_on_colaboratory": "display_on_colaboratory",
    "append_js_expression": "append_js_expression",
    "remove_children": "remove_children",
    "contains": "contains",
    "num_children": "num_children",
    "get_child_at": "get_child_at",
    "Int": "int_and_number",
    "Number": "int_and_number",
    "Float": "int_and_number",
    "String": "string",
    "Boolean": "boolean",
    "Bool": "boolean",
    "Array": "array",
    "Dictionary": "dictionary",
    "DisplayObject": "display_object",
    "get_css": "display_object_get_and_set_css",
    "set_css": "display_object_get_and_set_css",
    "fill_color": "graphics_base_fill_color",
    "fill_alpha": "graphics_base_fill_alpha",
    "line_color": "graphics_base_line_color",
    "line_alpha": "graphics_base_line_alpha",
    "line_thickness": "graphics_base_line_thickness",
    "line_dot_setting": "graphics_base_line_dot_setting",
    "line_dash_setting": "graphics_base_line_dash_setting",
    "line_round_dot_setting": "graphics_base_line_round_dot_setting",
    "line_dash_dot_setting": "graphics_base_line_dash_dot_setting",
    "rotation_around_center": "graphics_base_rotation_around_center",
    "get_rotation_around_point": "graphics_base_rotation_around_point",
    "set_rotation_around_point": "graphics_base_rotation_around_point",
    "scale_x_from_center": "graphics_base_scale_from_center",
    "scale_y_from_center": "graphics_base_scale_from_center",
    "get_scale_x_from_point": "graphics_base_scale_from_point",
    "get_scale_y_from_point": "graphics_base_scale_from_point",
    "set_scale_x_from_point": "graphics_base_scale_from_point",
    "set_scale_y_from_point": "graphics_base_scale_from_point",
    "flip_x": "graphics_base_flip_interfaces",
    "flip_y": "graphics_base_flip_interfaces",
    "skew_x": "graphics_base_skew",
    "skew_y": "graphics_base_skew",
    "Rectangle": "rectangle",
    "Circle": "circle",
    "Ellipse": "ellipse",
    "Line": "line",
    "Polyline": "polyline",
    "Polygon": "polygon",
    "Path": "path",
    "Graphics": "graphics",
    "begin_fill": "graphics_begin_fill",
    "line_style": "graphics_line_style",
    "draw_rect": "graphics_draw_rect",
    "draw_round_rect": "graphics_draw_round_rect",
    "draw_circle": "graphics_draw_circle",
    "draw_ellipse": "graphics_draw_ellipse",
    "draw_line": "graphics_draw_line",
    "draw_dotted_line": "graphics_draw_dotted_line",
    "draw_dashed_line": "graphics_draw_dashed_line",
    "draw_round_dotted_line": "graphics_draw_round_dotted_line",
    "draw_dash_dotted_line": "graphics_draw_dash_dotted_line",
    "draw_polygon": "graphics_draw_polygon",
    "draw_path": "graphics_draw_path",
    "Point2D": "point2d",
    "PathMoveTo": "path_move_to",
    "PathLineTo": "path_line_to",
    "PathHorizontal": "path_horizontal",
    "PathVertical": "path_vertical",
    "PathClose": "path_close",
    "PathBezier2D": "path_bezier_2d",
    "PathBezier2DContinual": "path_bezier_2d_continual",
    "PathBezier3D": "path_bezier_3d",
    "PathBezier3DContinual": "path_bezier_3d_continual",
    "prevent_default": "event_prevent_default_and_stop_propagation",
    "stop_propagation": "event_prevent_default_and_stop_propagation",
    "bind_custom_event": "bind_and_trigger_custom_event",
    "trigger_custom_event": "bind_and_trigger_custom_event",
    "MouseEvent": "mouse_event_abstract",
    "click": "click",
    "unbind_click": "click",
    "unbind_click_all": "click",
    "dblclick": "dblclick",
    "unbind_dblclick": "dblclick",
    "unbind_dblclick_all": "dblclick",
    "mousedown": "mousedown_and_mouseup",
    "mouseup": "mousedown_and_mouseup",
    "unbind_mousedown": "mousedown_and_mouseup",
    "unbind_mousedown_all": "mousedown_and_mouseup",
    "unbind_mouseup": "mousedown_and_mouseup",
    "unbind_mouseup_all": "mousedown_and_mouseup",
    "mouseover": "mouseover_and_mouseout",
    "mouseout": "mouseover_and_mouseout",
    "unbind_mouseover": "mouseover_and_mouseout",
    "unbind_mouseover_all": "mouseover_and_mouseout",
    "unbind_mouseout": "mouseover_and_mouseout",
    "unbind_mouseout_all": "mouseover_and_mouseout",
    "mousemove": "mousemove",
    "unbind_mousemove": "mousemove",
    "unbind_mousemove_all": "mousemove",
    "If": "if",
    "Elif": "elif",
    "Else": "else",
    "Return": "return",
    "For": "for",
    "Continue": "continue",
    "Timer": "timer",
    "TimerEvent": "timer_event",
    "FPS": "fps",
    "timer_complete": "timer_complete",
    "AnimationEvent": "animation_event",
    "animation_complete": "animation_complete",
    "animation_pause": "animation_pause_and_play",
    "animation_play": "animation_pause_and_play",
    "animation_reset": "animation_reset",
    "animation_finish": "animation_finish",
    "animation_reverse": "animation_reverse",
    "animation_time": "animation_time",
    "Easing": "easing_enum",
    "animation_parallel": "animation_parallel",
    "animation_move": "animation_move",
    "animation_x": "animation_x",
    "animation_y": "animation_y",
    "animation_width": "animation_width_and_height",
    "animation_height": "animation_width_and_height",
    "animation_fill_color": "animation_fill_color",
    "animation_fill_alpha": "animation_fill_alpha",
    "animation_line_color": "animation_line_color",
    "animation_line_alpha": "animation_line_alpha",
    "animation_line_thickness": "animation_line_thickness",
    "animation_radius": "animation_radius",
    "animation_rotation_around_center": "animation_rotation_around_center",
    "animation_rotation_around_point": "animation_rotation_around_point",
    "animation_scale_x_from_center": "animation_scale_x_and_y_from_center",
    "animation_scale_y_from_center": "animation_scale_x_and_y_from_center",
    "animation_scale_x_from_point": "animation_scale_x_and_y_from_point",
    "animation_scale_y_from_point": "animation_scale_x_and_y_from_point",
    "delete": "delete",
    "trace": "trace",
    "set_debug_mode": "set_debug_mode",
    "unset_debug_mode": "unset_debug_mode",
    "add_debug_info_setting": "add_debug_info_setting",
    "variable_name_suffix": "variable_name_suffix",
    "assert_equal": "assert_equal_and_not_equal",
    "assert_not_equal": "assert_equal_and_not_equal",
    "assert_true": "assert_true_and_false",
    "assert_false": "assert_true_and_false",
    "assert_arrays_equal": "assert_arrays_equal_and_arrays_not_equal",
    "assert_arrays_not_equal": "assert_arrays_equal_and_arrays_not_equal",
    "assert_dicts_equal": "assert_dicts_equal_and_dicts_not_equal",
    "assert_dicts_not_equal": "assert_dicts_equal_and_dicts_not_equal",
    "assert_defined": "assert_defined_and_undefined",
    "assert_undefined": "assert_defined_and_undefined",
    "assert_dicts_equal": "assert_dicts_equal_and_dicts_not_equal",
    "assert_dicts_not_equal": "assert_dicts_equal_and_dicts_not_equal",
    "assert_defined": "assert_defined_and_undefined",
    "assert_undefined": "assert_defined_and_undefined",
    "print": "trace",
    "ap.trace": "trace",
    "DateTime": "datetime",
    "assert_greater": "assert_greater_and_greater_equal",
    "assert_greater_equal": "assert_greater_and_greater_equal",
    "assert_less": "assert_less_and_less_equal",
    "assert_less_equal": "assert_less_and_less_equal",
}



"""Class implementation for the y animation value.
"""

from typing import Generic
from typing import TypeVar
from typing import Union

from typing_extensions import final

from apysc._animation.animation_base import AnimationBase
from apysc._animation.easing import Easing
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.variable_name_mixin import VariableNameMixIn

_T = TypeVar("_T", bound=VariableNameMixIn)


class AnimationY(AnimationBase[_T], Generic[_T]):
    """
    The animation class for a y-coordinate.

    References
    ----------
    - animation_y interface
        - https://simon-ritchie.github.io/apysc/en/animation_y.html
    - Animation interfaces duration setting
        - https://simon-ritchie.github.io/apysc/en/animation_duration.html
    - Animation interfaces delay setting
        - https://simon-ritchie.github.io/apysc/en/animation_delay.html
    - Each animation interface return value
        - https://simon-ritchie.github.io/apysc/en/animation_return_value.html  # noqa
    - Sequential animation setting
        - https://simon-ritchie.github.io/apysc/en/sequential_animation.html
    - animation_parallel interface
        - https://simon-ritchie.github.io/apysc/en/animation_parallel.html
    - Easing enum
        - https://simon-ritchie.github.io/apysc/en/easing_enum.html

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color="#0af")
    >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
    ...     x=50, y=50, width=50, height=50
    ... )
    >>> animation: ap.AnimationY = rectangle.animation_y(
    ...     y=100,
    ...     duration=1500,
    ...     easing=ap.Easing.EASE_OUT_QUINT,
    ... )
    >>> _ = animation.start()
    """

    _y: Int

    @final
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        *,
        target: _T,
        y: Union[int, Int],
        duration: Union[int, Int] = 3000,
        delay: Union[int, Int] = 0,
        easing: Easing = Easing.LINEAR,
    ) -> None:
        """
        The animation class for a y-coordinate.

        Parameters
        ----------
        target : VariableNameMixIn
            A target instance of the animation target
            (e.g., `DisplayObject` instance).
        y : int or Int
            Destination of the y-coordinate.
        duration : int or Int, default 3000
            Milliseconds before an animation ends.
        delay : int or Int, default 0
            Milliseconds before an animation starts.
        easing : Easing, default Easing.LINEAR
            Easing setting.
        """
        from apysc._converter import to_apysc_val_from_builtin
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.ANIMATION_Y
        )
        self._y = to_apysc_val_from_builtin.get_copied_int_from_builtin_val(integer=y)
        self._set_basic_animation_settings(
            target=target, duration=duration, delay=delay, easing=easing
        )
        super(AnimationY, self).__init__(variable_name=variable_name)

    @final
    def _get_animation_func_expression(self) -> str:
        """
        Get a animation function expression.

        Returns
        -------
        expression : str
            Animation function expression.
        """
        from apysc._type import value_util

        y_str: str = value_util.get_value_str_for_expression(value=self._y)
        return f"\n  .y({y_str});"

    @final
    def _get_complete_event_in_handler_head_expression(self) -> str:
        """
        Get an expression to insert into the complete event
        handler's head.

        Returns
        -------
        expression : str
            An expression to insert into the complete event
            handler's head.
        """
        from apysc._display.y_mixin import YMixIn

        expression: str = ""
        if isinstance(self._target, YMixIn):
            self._target._initialize_y_if_not_initialized()
            expression = (
                f"{self._target._y.variable_name} = " f"{self._y.variable_name};"
            )
        return expression



"""This module is for the translation mapping data of the
following document:

Document file: animation_complete.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# AnimationBase class animation_complete interface": "# AnimationBase クラス animation_complete インターフェイス",  # noqa
    ##################################################
    "This page explains the `AnimationBase` class `animation_complete` method interface.": "このページでは`AnimationBase`クラスの`animation_complete`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `animation_complete` method binds a handler that the animation calls when its end.": "`animation_complete`メソッドはアニメーションが終了したときのハンドラを設定します。",  # noqa
    ##################################################
    "The handler's arguments require the event instance (`ap.AnimationEvent`) at the first argument and the options dictionary at the second argument.": "ハンドラの引数には第一引数にイベントのインスタンス（`ap.AnimationEvent`）、第二引数にはオプションのパラメーターとなる辞書が必要になります。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `animation_complete` method requires a handler at the first argument and the optional options dictionary at the second argument.": "`animation_complete`メソッドは第一引数にハンドラが必要となり、第二引数にはオプションのパラメーターの辞書を設定することができます。",  # noqa
    ##################################################
    "The following example calls the `animation_complete` method at the x-coordinate animation end. It starts another animation to reset the x-coordinate:": "以下のコード例ではX座標のアニメーション終了時用に`animation_complete`メソッドを読んでハンドラを設定しています。そのハンドラ内ではX座標をリセットするための別のアニメーションを開始しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nDURATION: int = 1000\n\n\ndef on_animation_complete_1(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_x: ap.AnimationX = rectangle.animation_x(x=50, duration=DURATION)\n    animation_x.animation_complete(on_animation_complete_2)\n    animation_x.start()\n\n\ndef on_animation_complete_2(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_x: ap.AnimationX = rectangle.animation_x(x=100, duration=DURATION)\n    animation_x.animation_complete(on_animation_complete_1)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100, duration=1000)\nanimation_x.animation_complete(on_animation_complete_1)\nanimation_x.start()\n\nap.save_overall_html(dest_dir_path="./animation_complete_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nDURATION: int = 1000\n\n\ndef on_animation_complete_1(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_x: ap.AnimationX = rectangle.animation_x(x=50, duration=DURATION)\n    animation_x.animation_complete(on_animation_complete_2)\n    animation_x.start()\n\n\ndef on_animation_complete_2(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_x: ap.AnimationX = rectangle.animation_x(x=100, duration=DURATION)\n    animation_x.animation_complete(on_animation_complete_1)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100, duration=1000)\nanimation_x.animation_complete(on_animation_complete_1)\nanimation_x.start()\n\nap.save_overall_html(dest_dir_path="./animation_complete_basic_usage/")\n```',  # noqa
    ##################################################
    "## Notes about the other interface calling order": "## 他のインターフェイスを呼び出す際の特記事項",
    ##################################################
    "You can only call the `animation_complete` before the animation start, so if you call the `animation_complete` method after the `start` method, it raises an exception:": "`animation_complete`メソッドはアニメーション開始前にのみ設定することができます。`start`メソッド呼び出し後に`animation_complete`メソッドを呼び出すとエラーになります。",  # noqa
    ##################################################
    '```py\nimport apysc as ap\n\n\ndef on_animation_complete(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace("Animation complete!")\n\n\nap.Stage(\n    stage_width=200, stage_height=200, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nanimation_move: ap.AnimationMove = rectangle.animation_move(x=100, y=100, duration=1000)\nanimation_move.start()\nanimation_move.animation_complete(on_animation_complete)\n```': '```py\nimport apysc as ap\n\n\ndef on_animation_complete(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace("Animation complete!")\n\n\nap.Stage(\n    stage_width=200, stage_height=200, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nanimation_move: ap.AnimationMove = rectangle.animation_move(x=100, y=100, duration=1000)\nanimation_move.start()\nanimation_move.animation_complete(on_animation_complete)\n```',  # noqa
    ##################################################
    "```\nException: This interface can not be called after the animation is started.\n```": "```\nException: This interface can not be called after the animation is started.\n```",  # noqa
    ##################################################
    "The calling of the `animation_complete` method before the `start` method works correctly:": "`start`メソッド呼び出し前に`animation_complete`メソッドを呼び出すことで正常に動作します:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace("Animation complete!")\n\n\nap.Stage(\n    stage_width=200, stage_height=200, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nanimation_move: ap.AnimationMove = rectangle.animation_move(x=100, y=100, duration=1000)\nanimation_move.animation_complete(on_animation_complete)\nanimation_move.start()\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace("Animation complete!")\n\n\nap.Stage(\n    stage_width=200, stage_height=200, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nanimation_move: ap.AnimationMove = rectangle.animation_move(x=100, y=100, duration=1000)\nanimation_move.animation_complete(on_animation_complete)\nanimation_move.start()\n```',  # noqa
    ##################################################
    "## animation_complete API": "## animation_complete API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Add an animation complete event listener setting.<hr>": "アニメーション終了時のイベントリスナーの設定を追加します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - A callable that an instance calls when an animation is complete.": "  - アニメーション終了時に実行される関数もしくはメソッド。",  # noqa
    ##################################################
    "- `options`: dict or None, default None": "- `options`: dict or None, default None",  # noqa
    ##################################################
    "  - Optional arguments dictionary to be passed to a handler.": "  - ハンドラに渡される省略が可能な追加のパラメーターとしての辞書。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `self`: AnimatonBase": "- `self`: AnimatonBase",
    ##################################################
    "  - This instance.": "  - このインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Raises]**": "**[エラー発生条件]**",
    ##################################################
    "- Exception: If calling this interface after an animation starts": "- Exception: もしアニメーション開始後にこのインターフェイスを呼び出している場合。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "This interface can only use before an animation starts<hr>": "このインターフェイスはアニメーション開始前にのみ利用することができます。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_animation_complete(\n...     e: ap.AnimationEvent[ap.Rectangle], options: dict\n... ) -> None:\n...     ap.trace("Animation completed!")\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = (\n...     rectangle.animation_x(\n...         x=100,\n...     )\n...     .animation_complete(on_animation_complete)\n...     .start()\n... )\n```': '```py\n>>> import apysc as ap\n>>> def on_animation_complete(\n...     e: ap.AnimationEvent[ap.Rectangle], options: dict\n... ) -> None:\n...     ap.trace("Animation completed!")\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = (\n...     rectangle.animation_x(\n...         x=100,\n...     )\n...     .animation_complete(on_animation_complete)\n...     .start()\n... )\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [About the handler options' type](https://simon-ritchie.github.io/apysc/en/about_handler_options_type.html)": "- [ハンドラのoptions引数の型について](https://simon-ritchie.github.io/apysc/jp/jp_about_handler_options_type.html)",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: append_js_expression.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# append_js_expression interface": "# append_js_expression インターフェイス",
    ##################################################
    "This page explains the `append_js_expression` function interface.": "このページでは`append_js_expression`関数のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `append_js_expression` function interface appends any JavaScript to the exported HTML at any position. This interface may be helpful if you need to use the apysc not supported interfaces or irregular JavaScript implementation, like the Django template tags or parameters (e.g., `{% if ... %}`).": "`append_js_expression`関数は出力先のHTMLの任意の場所にJavaScriptのコードを追加します。このインターフェイスはapyscがサポートしていない特殊な処理などを追加する際などに役に立つことがあります（Djangoなどのライブラリでテンプレートタグなどを独自の出力したい場合など）。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `append_js_expression` function requires JavaScript string at the argument.": "`append_js_expression`関数は引数にJavaScriptのコードの文字列が必要とします。",  # noqa
    ##################################################
    "The following example appends the `console.log` JavaScript calling at the rectangle click handler:": "以下のコード例では四角をクリックした際のハンドラ内で`console.log`のJavaScriptの関数呼び出しのコードを追加しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.append_js_expression(expression=\'console.log("The rectangle is clicked!");\')\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="append_js_expression_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.append_js_expression(expression=\'console.log("The rectangle is clicked!");\')\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="append_js_expression_basic_usage/")\n```',  # noqa
    ##################################################
    "If you click the following rectangle, the handler displays the `The rectangle is clicked!` message on the browser console (please press the F12 key).": "四角をクリックすると`The rectangle is clicked!`というメッセージがブラウザのコンソールに表示されます（F12キーを押して確認してください）。",  # noqa
    ##################################################
    "## append_js_expression API": "## append_js_expression API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Append js expression.<hr>": "JavaScriptのコードの追加を行います。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `expression`: str": "- `expression`: str",
    ##################################################
    "  - JavaScript Expression string.": "  - 追加対象のJavaScriptのコードの文字列。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> ap.append_js_expression(expression='console.log(\"Hello!\")')\n```": "```py\n>>> import apysc as ap\n>>> ap.append_js_expression(expression='console.log(\"Hello!\")')\n```",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: animation_delay.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Animation interfaces delay setting": "# 各アニメーションのインターフェイスのdelayの設定",
    ##################################################
    "This page explains the animation interfaces `delay` setting.": "このページでは各アニメーションのインターフェイスの`delay`の設定について説明します。",  # noqa
    ##################################################
    "## What setting is this?": "## 設定概要",
    ##################################################
    "The `delay` setting determines the delay time before the animation starts. For instance, if you specify 3000 to the `delay` argument, the animation starts after 3 seconds after.": "`delay`の設定はアニメーション開始前の遅延時間を値を設定することができます。たとえば`delay`の引数に3000を指定すればアニメーションは3秒後に開始します。",  # noqa
    ##################################################
    "Each animation method interface (such as the `animation_move`\\, `animation_x`\\, and so on) has the `delay` argument.": "`animation_move`や`animation_x`などの各アニメーションのインターフェイスはこの`delay`引数を持っています。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The following example sets 2 seconds between each x-coordinate animation (pause 2 seconds before animation starts):": "以下の例では各X座標のアニメーション間で2秒の遅延設定(delay)を行っています。2秒間停止してからアニメーションが開始します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nDURATION: int = 3000\nDELAY: int = 2000\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_x: ap.AnimationX = rectangle.animation_x(\n        x=50, duration=DURATION, delay=DELAY, easing=EASING\n    )\n    animation_x.animation_complete(on_animation_complete_2)\n    animation_x.start()\n\n\ndef on_animation_complete_2(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_x: ap.AnimationX = rectangle.animation_x(\n        x=300, duration=DURATION, delay=DELAY, easing=EASING\n    )\n    animation_x.animation_complete(on_animation_complete_1)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=400, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nanimation_x: ap.AnimationX = rectangle.animation_x(\n    x=300, duration=DURATION, delay=DELAY, easing=EASING\n)\nanimation_x.animation_complete(on_animation_complete_1)\nanimation_x.start()\n\nap.save_overall_html(dest_dir_path="./animation_delay_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nDURATION: int = 3000\nDELAY: int = 2000\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_x: ap.AnimationX = rectangle.animation_x(\n        x=50, duration=DURATION, delay=DELAY, easing=EASING\n    )\n    animation_x.animation_complete(on_animation_complete_2)\n    animation_x.start()\n\n\ndef on_animation_complete_2(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_x: ap.AnimationX = rectangle.animation_x(\n        x=300, duration=DURATION, delay=DELAY, easing=EASING\n    )\n    animation_x.animation_complete(on_animation_complete_1)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=400, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nanimation_x: ap.AnimationX = rectangle.animation_x(\n    x=300, duration=DURATION, delay=DELAY, easing=EASING\n)\nanimation_x.animation_complete(on_animation_complete_1)\nanimation_x.start()\n\nap.save_overall_html(dest_dir_path="./animation_delay_basic_usage/")\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: graphics_base_fill_color.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# GraphicsBase fill_color interface": "# GraphicsBase クラスの fill_color インターフェイス",
    ##################################################
    "This page explains the `GraphicsBase` class `fill_color` property interface.": "このページでは`GraphicsBase`クラスの`fill_color`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `fill_color` property interface updates or get the instance's fill color.": "`fill_color`属性のインターフェイスでは塗りの色の値を更新したり取得したりすることができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The getter interface becomes the `String` hex color code value, and the setter one also requires the `String` hex color code value.": "getterのインターフェイスは`String`型の16進数のカラーコードの文字列になります。setterのインターフェイスも`String`型の16進数のカラーコードの指定が必要になります。",  # noqa
    ##################################################
    "The following example changes the fill color (from cyan to magenta and magenta to cyan) when you click the rectangle:": "以下のコード例では四角をクリックした際に塗りの色をシアンからマゼンタに変更するようにしています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    fill_color: ap.String = rectangle.fill_color\n    with ap.If(fill_color == "#00aaff"):\n        rectangle.fill_color = ap.String("#f0a")\n    with ap.Else():\n        rectangle.fill_color = ap.String("#0af")\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="./graphics_base_fill_color_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    fill_color: ap.String = rectangle.fill_color\n    with ap.If(fill_color == "#00aaff"):\n        rectangle.fill_color = ap.String("#f0a")\n    with ap.Else():\n        rectangle.fill_color = ap.String("#0af")\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(dest_dir_path="./graphics_base_fill_color_basic_usage/")\n```',  # noqa
    ##################################################
    "## fill_color property API": "## fill_color 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get this instance's fill color.<hr>": "インスタンスの塗りの色を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `fill_color`: String": "- `fill_color`: String",
    ##################################################
    "  - Current fill color (hexadecimal string, e.g., '#00aaff'). If not be set, this interface returns a blank string.": "  - 現在の塗りの色（`'#00aaff'`などの16進数の文字列）。もしも設定されていない場合空文字が返却されます。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.fill_color = ap.String("#f0a")\n>>> rectangle.fill_color\nString(\'#ff00aa\')\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.fill_color = ap.String("#f0a")\n>>> rectangle.fill_color\nString(\'#ff00aa\')\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: assert_greater_and_greater_equal.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# assert_greater and assert_greater_equal interfaces": "# assert_greater と assert_greater_equal の各インターフェイス",  # noqa
    ##################################################
    "This page explains the `assert_greater` and `assert_greater_equal` function interfaces.": "このページでは`assert_greater`と`assert_greater_equal`関数の各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `assert_greater` function interface asserts a specified first value is greater than a specified second value.": "`assert_greater`関数のインターフェイスでは1つ目に指定された値が2つ目に指定された値よりも大きいことをチェックします。",  # noqa
    ##################################################
    "Similarly, the `assert_greater_equal` function interface asserts a specified first value is greater than or equal to a specified second value.": "似たような形で、`assert_greater_equal`関数では最初に指定された値が2つ目の値よりも大きいかもしくは同値なことをチェックします。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [JavaScript assertion interface basic behavior](assertion_basic_behavior.md)": "- [JavaScriptの各アサーションのインターフェイスの基本的な挙動](jp_assertion_basic_behavior.md)",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `assert_greater` and `assert_greater_equal` interfaces require the `left` and `right` arguments.": "`assert_greater`と`assert_greater_equal`インターフェイスは`left`と`right`の引数を必要とします。",  # noqa
    ##################################################
    "These arguments only accept numeric values, such as the Python built-in `int`, `float`, apysc `Int`, or `Number` value.": "これらの引数はPythonビルトインの`int`や`float`、apyscの`Int`や`Number`などの数値の値のみ受け付けます。",  # noqa
    ##################################################
    "The `msg` argument is optional.": "`msg`引数は省略可です。",
    ##################################################
    "This interface displays a specified `msg` (message) argument to the browser console when an assertion fails.": "このインターフェイスはアサーションが失敗した際に`msg`（message）引数の値をブラウザのコンソール上に表示します。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\nint_val_1: ap.Int = ap.Int(10)\nint_val_2: ap.Int = ap.Int(9)\nap.assert_greater(left=int_val_1, right=int_val_2, msg="Assertion failed")\n\nint_val_3: ap.Int = ap.Int(10)\nap.assert_greater_equal(left=int_val_1, right=int_val_3, msg="Assertion failed")\n\nap.save_overall_html(dest_dir_path="assert_greater_and_greater_equal_basic_usage_1/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\nint_val_1: ap.Int = ap.Int(10)\nint_val_2: ap.Int = ap.Int(9)\nap.assert_greater(left=int_val_1, right=int_val_2, msg="Assertion failed")\n\nint_val_3: ap.Int = ap.Int(10)\nap.assert_greater_equal(left=int_val_1, right=int_val_3, msg="Assertion failed")\n\nap.save_overall_html(dest_dir_path="assert_greater_and_greater_equal_basic_usage_1/")\n```',  # noqa
    ##################################################
    "The following example fails an assertion and displays the `Assertion failed` message on the browser console:": "`以下の例ではアサーションが失敗し、`Assertion failed`というメッセージがブラウザ上のコンソールに表示されます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\nint_val_1: ap.Int = ap.Int(9)\nint_val_2: ap.Int = ap.Int(10)\nap.assert_greater(left=int_val_1, right=int_val_2, msg="Assertion failed")\n\nap.save_overall_html(dest_dir_path="assert_greater_and_greater_equal_basic_usage_2/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=0, stage_height=0, background_color="#333", stage_elem_id="stage"\n)\nint_val_1: ap.Int = ap.Int(9)\nint_val_2: ap.Int = ap.Int(10)\nap.assert_greater(left=int_val_1, right=int_val_2, msg="Assertion failed")\n\nap.save_overall_html(dest_dir_path="assert_greater_and_greater_equal_basic_usage_2/")\n```',  # noqa
    ##################################################
    "## assert_greater API": "## assert_greater のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "JavaScript assertion interface for the greater than condition.<hr>": "JavaScriptの超過条件のアサーションのためのインターフェイスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `left`: Union[int, float, Int, Number]": "- `left`: Union[int, float, Int, Number]",  # noqa
    ##################################################
    "  - Left-side (greater) value to compare.": "  - 比較用の左辺の値（大きい側の値）。",
    ##################################################
    "- `right`: Union[int, float, Int, Number]": "- `right`: Union[int, float, Int, Number]",  # noqa
    ##################################################
    "  - Right-side (less) value to compare.": "  - 比較用の右辺の値（小さい側の値）。",
    ##################################################
    "- `msg`: str, optional": "- `msg`: str, optional",
    ##################################################
    "  - Message to display when assertion failed.": "  - チェックに失敗した際に表示するメッセージ。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> int_val_1: ap.Int = ap.Int(10)\n>>> int_val_2: ap.Int = ap.Int(9)\n>>> ap.assert_greater(left=int_val_1, right=int_val_2)\n```": "```py\n>>> import apysc as ap\n>>> int_val_1: ap.Int = ap.Int(10)\n>>> int_val_2: ap.Int = ap.Int(9)\n>>> ap.assert_greater(left=int_val_1, right=int_val_2)\n```",  # noqa
    ##################################################
    "## assert_greater_equal API": "## assert_greater_equal のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "JavaScript assertion interface for the greater than or equal to condition.<hr>": "JavaScriptの以上の条件のアサーションのためのインターフェイスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `left`: Union[int, float, Int, Number]": "- `left`: Union[int, float, Int, Number]",  # noqa
    ##################################################
    "  - Left-side (greater) value to compare.": "  - 比較用の左辺の値（大きい側の値）。",
    ##################################################
    "- `right`: Union[int, float, Int, Number]": "- `right`: Union[int, float, Int, Number]",  # noqa
    ##################################################
    "  - Right-side (less) value to compare.": "  - 比較用の右辺の値（小さい側の値）。",
    ##################################################
    "- `msg`: str, optional": "- `msg`: str, optional",
    ##################################################
    "  - Message to display when assertion failed.": "  - チェックに失敗した際に表示するメッセージ。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> int_val_1: ap.Int = ap.Int(10)\n>>> int_val_2: ap.Int = ap.Int(9)\n>>> ap.assert_greater_equal(left=int_val_1, right=int_val_2)\n>>> int_val_3: ap.Int = ap.Int(10)\n>>> ap.assert_greater_equal(left=int_val_1, right=int_val_3)\n```": "```py\n>>> import apysc as ap\n>>> int_val_1: ap.Int = ap.Int(10)\n>>> int_val_2: ap.Int = ap.Int(9)\n>>> ap.assert_greater_equal(left=int_val_1, right=int_val_2)\n>>> int_val_3: ap.Int = ap.Int(10)\n>>> ap.assert_greater_equal(left=int_val_1, right=int_val_3)\n```",  # noqa
}



"""String's validation implementations.
"""

from typing import Union

from apysc._type.string import String


def validate_string_type(
    *, string: Union[str, String], additional_err_msg: str = ""
) -> None:
    """
    Validate a specified string's type is str.

    Parameters
    ----------
    string : String or str
        A string to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        - If a non-string value is specified.
    """
    if isinstance(string, str):
        return
    if isinstance(string, String):
        return
    if additional_err_msg != "":
        additional_err_msg = f"\n{additional_err_msg}"
    raise ValueError(
        f"A specified value is not str type: {type(string)}" f"{additional_err_msg}"
    )


def validate_builtin_string_type(*, string: str, additional_err_msg: str = "") -> None:
    """
    Validate a specified string's type is Python built-in's str.

    Parameters
    ----------
    string : str
        A string to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        - If a non-string value is specified.
    """
    if isinstance(string, str):
        return
    if additional_err_msg != "":
        additional_err_msg = f"\n{additional_err_msg}"
    raise ValueError(
        f"A specified value is not str type: {type(string)}" f"{additional_err_msg}"
    )


def validate_not_empty_string(
    *, string: Union[str, String], additional_err_msg: str = ""
) -> None:
    """
    Validate whether a specified string is not empty.

    Parameters
    ----------
    string : String or str
        String to check.
    additional_err_msg : str, optional
        An additional error message to display.

    Raises
    ------
    ValueError
        - If empty string ('' or "") is specified.
        - If specified value is not str type.
    """
    validate_string_type(string=string)
    if isinstance(string, str):
        if string != "":
            return
    elif isinstance(string, String):
        if string._value != "":
            return
    if additional_err_msg != "":
        additional_err_msg = f"\n{additional_err_msg}"
    raise ValueError(f"Empty string is not acceptable.{additional_err_msg}")



"""This module is for the translation mapping data of the
following document:

Document file: array_extend_and_concat.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Array class extend and concat interfaces": "# Array クラスの extend と concat のインターフェイス",  # noqa
    ##################################################
    "This page explains the `Array` class `extend` and `concat` method interfaces.": "このページでは`Array`クラスの`extend`と`concat`メソッドの各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `extend` and `concat` method interfaces are the two arrays' concatenation interfaces.": "`extend`と`concat`メソッドの各インターフェイスは2つの配列の連結処理を扱います。",  # noqa
    ##################################################
    "The `extend` method updates an original array in place and returns the `None`. The `concat` method returns the concatenated array, and an original one is not updated.": "`extend`メソッドは元々の配列自体を更新し返却値は設定されずNoneとなります。`concat`メソッドでは連結結果の配列を返却します。元の配列は変化しません。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `extend` and `concat` methods require other iterable objects, like the `list`\\, `tuple`\\, or `Array` value at the first argument, as follows:": "`extend`と`concat`メソッドは以下のコード例のようにそれぞれ第一引数に連結対象の他の配列など（ビルトインのリストやタプル、apyscの`Array`など）のオブジェクトを必要とします:",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2])\narr.extend([3, 4])\nassert arr == [1, 2, 3, 4]\n\nother_arr: ap.Array[int] = arr.concat([5, 6])\nassert other_arr == [1, 2, 3, 4, 5, 6]\nassert arr == [1, 2, 3, 4]\n```": "```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2])\narr.extend([3, 4])\nassert arr == [1, 2, 3, 4]\n\nother_arr: ap.Array[int] = arr.concat([5, 6])\nassert other_arr == [1, 2, 3, 4, 5, 6]\nassert arr == [1, 2, 3, 4]\n```",  # noqa
    ##################################################
    "## extend API": "## extend API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Concatenate argument array to this one. This interface positions the argument array's values after this array values. This method is similar to the concat method. Still, there is a difference in whether updating the same variable (extend) or returning as a different variable (concat).<hr>": "引数に指定された配列をこの配列へ連結します。このインターフェイスは引数の配列の値をこの配列の後に配置します。このメソッドはconcatメソッドと似た挙動をしますが、配列の値を直接更新するか（extend）もしくは別の値として返却するか（concat）の違いがあります。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `other_arr`: Array or list or tuple or range": "- `other_arr`: Array or list or tuple or range",  # noqa
    ##################################################
    "  - Other array-like values to concatenate.": "  - 連結対象となる他の配列の（もしくはそれに近しい）値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.extend([4, 5, 6])\n>>> arr\nArray([1, 2, 3, 4, 5, 6])\n```": "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.extend([4, 5, 6])\n>>> arr\nArray([1, 2, 3, 4, 5, 6])\n```",  # noqa
    ##################################################
    "## concat API": "## concat API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Concatenate argument array to this one. This interface positions the argument array's values after this array values. This method is similar to extend method, but there is a difference in whether updating the same variable (extend) or returning as a different variable (concat).<hr>": "引数に指定された配列をこの配列へ連結します。このインターフェイスは引数の配列の値をこの配列の後に配置します。このメソッドはextendメソッドと似た挙動をしますが、配列の値を直接更新するか（extend）もしくは別の値として返却するか（concat）の違いがあります。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `other_arr`: Array or list or tuple": "- `other_arr`: Array or list or tuple",
    ##################################################
    "  - Other array-like values to concatenate.": "  - 連結対象となる他の配列の（もしくはそれに近しい）値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `concatenated`: Array": "- `concatenated`: Array",
    ##################################################
    "  - Concatenated array value.": "  - 連結結果の配列の値。.",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr = arr.concat([4, 5, 6])\n>>> arr\nArray([1, 2, 3, 4, 5, 6])\n```": "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr = arr.concat([4, 5, 6])\n>>> arr\nArray([1, 2, 3, 4, 5, 6])\n```",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: dictionary_get.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Dictionary get interface": "# Dictionary クラスの get インターフェイス",
    ##################################################
    "This page explains the `Dictionary` class `get` method interface.": "このページでは`Dictionary`クラスの`get`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `get` method returns the specified key's value. If that key does not exist in the dictionary, it returns the default value (not raising an exception).": "`get`メソッドは引数に指定されたキーの値を返却します。もし指定されたキーが辞書内に存在しなければデフォルトの値を返却します（キーが無くともエラーにはなりません）。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `get` method requires the first argument, `key` (dictionary key). The second argument of the `default` is optional, and if not provided, it returns the `None` value.": "`get`メソッドは`key`（辞書のキー）の第一引数を必要とします。第二引数の`default`引数は省略可で、もし指定されなければ`None`がデフォルト値となります。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing import Any, Optional\n\nimport apysc as ap\n\ndict_val: ap.Dictionary = ap.Dictionary({"a": 10})\ngot_val_1: int = dict_val.get(key="a", default=0)\nassert got_val_1 == 10\n\ngot_val_2: int = dict_val.get(key="b", default=0)\nassert got_val_2 == 0\n\ngot_val_3: Optional[Any] = dict_val.get(key="b")\nassert got_val_3 is None\n```': '```py\n# runnable\nfrom typing import Any, Optional\n\nimport apysc as ap\n\ndict_val: ap.Dictionary = ap.Dictionary({"a": 10})\ngot_val_1: int = dict_val.get(key="a", default=0)\nassert got_val_1 == 10\n\ngot_val_2: int = dict_val.get(key="b", default=0)\nassert got_val_2 == 0\n\ngot_val_3: Optional[Any] = dict_val.get(key="b")\nassert got_val_3 is None\n```',  # noqa
    ##################################################
    "## get API": "## get API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a specified key dictionary value. If this dictionary hasn't a specified key, this interface returns a default value.<hr>": "指定されたキーの辞書内の値を取得します。もし指定されたキーが辞書に存在しない場合、このインターフェイスはデフォルト値を返却します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `key`: _K": "- `key`: _K",
    ##################################################
    "  - Target key.": "  - 対象のキー。",
    ##################################################
    "- `default`: DefaultType or None, optional": "- `default`: DefaultType or None, optional",  # noqa
    ##################################################
    "  - Any default value.": "  - 任意のデフォルト値の値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `result_value`: Any": "- `result_value`: Any",
    ##################################################
    "  - Extracted value or a default value.": "  - 抽出された値もしくはデフォルト値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> from typing import Optional\n>>> import apysc as ap\n>>> dictionary: ap.Dictionary = ap.Dictionary({"a": 10})\n>>> value_1: Optional[int] = dictionary.get("a")\n>>> value_1\n10\n\n>>> value_2: Optional[int] = dictionary.get("b")\n>>> print(value_2)\nNone\n\n>>> value_3: int = dictionary.get("c", default=0)\n>>> value_3\n0\n```': '```py\n>>> from typing import Optional\n>>> import apysc as ap\n>>> dictionary: ap.Dictionary = ap.Dictionary({"a": 10})\n>>> value_1: Optional[int] = dictionary.get("a")\n>>> value_1\n10\n\n>>> value_2: Optional[int] = dictionary.get("b")\n>>> print(value_2)\nNone\n\n>>> value_3: int = dictionary.get("c", default=0)\n>>> value_3\n0\n```',  # noqa
}



"""Class implementation for the stop_propagation mix-in.
"""

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.variable_name_mixin import VariableNameMixIn


class StopPropagationMixIn(VariableNameMixIn):
    @final
    @add_debug_info_setting(module_name=__name__)
    def stop_propagation(self) -> None:
        """
        Stop an event propagation.

        References
        ----------
        - Event class prevent_default and stop_propagation interfaces
            - https://simon-ritchie.github.io/apysc/en/event_prevent_default_and_stop_propagation.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> def on_click(e: ap.MouseEvent, options: dict) -> None:
        ...     e.stop_propagation()
        ...     ap.trace("Clicked!")
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color="#0af")
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> _ = sprite.click(on_click)
        >>> _ = rectangle.click(on_click)
        """
        import apysc as ap

        expression: str = f"{self.variable_name}.stopPropagation();"
        ap.append_js_expression(expression=expression)



"""This module is for the interface base class of setting
an x and y coordinate with a minimum point.
"""

from abc import ABC
from abc import abstractmethod


class SetXAndYWithMinimumPointInterfaceBase(ABC):
    @abstractmethod
    def _set_x_and_y_with_minimum_point(self) -> None:
        """
        Set an x and y properties coordinate with a minimum point.
        """



"""This module is for the translation mapping data of the
following document:

Document file: display_object_visible.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# DisplayObject class visible interface": "# DisplayObject クラスの visible インターフェイス",
    ##################################################
    "This page explains the `DisplayObject` class `visible` property interface.": "このページでは`DisplayObject`クラスの`visible`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `visible` property interface will change the `DisplayObject` visible / invisible state.": "`visible`属性は`DisplayObject`の表示・非表示の状態を切り替えます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `visible` property accepts a `Boolean` value. If you set the True value, a `DisplayObject` instance becomes visible (default). Conversely, if you set the False value, a `DisplayObject` instance becomes invisible.": "`visible`属性は`Boolean`の値を受け付けます。Trueを設定した場合その`DisplayObject`のインスタンスは表示状態になります（デフォルトの状態になります）。Falseを設定するとその`DisplayObject`のインスタンスは非表示になります。",  # noqa
    ##################################################
    "The following example switches the visible values when you click the rectangle. For example, suppose you click the left rectangle (the rectangle_1). In that case, the left rectangle becomes invisible, and the right rectangle (rectangle_2) becomes visible.": "以下のコード例では四角をクリックした時にvisibleの設定を切り替えています。左側の四角をクリックした際には左側の四角は非表示となり、右側に別の四角が表示されます。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_rectangle_1_click(e: ap.MouseEvent[ap.Rectangle], options: _RectOptions) -> None:\n    """\n    The handler that the first rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle_1: ap.Rectangle = e.this\n    rectangle_2: ap.Rectangle = options["rectangle"]\n    rectangle_1.visible = ap.Boolean(False)\n    rectangle_2.visible = ap.Boolean(True)\n\n\ndef on_rectangle_2_click(e: ap.MouseEvent[ap.Rectangle], options: _RectOptions) -> None:\n    """\n    The handler that the second rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle_1: ap.Rectangle = options["rectangle"]\n    rectangle_2: ap.Rectangle = e.this\n    rectangle_1.visible = ap.Boolean(True)\n    rectangle_2.visible = ap.Boolean(False)\n\n\nap.Stage(\n    background_color="#333", stage_width=250, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color="#0af")\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nsprite.graphics.begin_fill(color="#f0a")\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\nrectangle_2.visible = ap.Boolean(False)\n\noptions: _RectOptions = {"rectangle": rectangle_2}\nrectangle_1.click(on_rectangle_1_click, options=options)\noptions = {"rectangle": rectangle_1}\nrectangle_2.click(on_rectangle_2_click, options=options)\n\nap.save_overall_html(dest_dir_path="display_object_visible_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_rectangle_1_click(e: ap.MouseEvent[ap.Rectangle], options: _RectOptions) -> None:\n    """\n    The handler that the first rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle_1: ap.Rectangle = e.this\n    rectangle_2: ap.Rectangle = options["rectangle"]\n    rectangle_1.visible = ap.Boolean(False)\n    rectangle_2.visible = ap.Boolean(True)\n\n\ndef on_rectangle_2_click(e: ap.MouseEvent[ap.Rectangle], options: _RectOptions) -> None:\n    """\n    The handler that the second rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle_1: ap.Rectangle = options["rectangle"]\n    rectangle_2: ap.Rectangle = e.this\n    rectangle_1.visible = ap.Boolean(True)\n    rectangle_2.visible = ap.Boolean(False)\n\n\nap.Stage(\n    background_color="#333", stage_width=250, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color="#0af")\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nsprite.graphics.begin_fill(color="#f0a")\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\nrectangle_2.visible = ap.Boolean(False)\n\noptions: _RectOptions = {"rectangle": rectangle_2}\nrectangle_1.click(on_rectangle_1_click, options=options)\noptions = {"rectangle": rectangle_1}\nrectangle_2.click(on_rectangle_2_click, options=options)\n\nap.save_overall_html(dest_dir_path="display_object_visible_basic_usage/")\n```',  # noqa
    ##################################################
    "## visible property API": "## visible 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a visibility value of this instance.<hr>": "このインスタンスの可視状態かどうかの値を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `result`: Boolean": "- `result`: Boolean",
    ##################################################
    "  - If this instance is visible, this interface returns True.": "  - もしこのインスタンスが表示状態であればTrueとなります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.visible = ap.Boolean(False)\n>>> rectangle.visible\nBoolean(False)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle.visible = ap.Boolean(False)\n>>> rectangle.visible\nBoolean(False)\n```',  # noqa
}



"""Class implementation for fill color mix-in.
"""

from typing import Dict
from typing import Union

from typing_extensions import final

from apysc._animation.animation_fill_color_mixin import AnimationFillColorMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_linking_mixin import AttrLinkingMixIn
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.string import String
from apysc._type.variable_name_suffix_attr_mixin import VariableNameSuffixAttrMixIn
from apysc._validation import arg_validation_decos


class FillColorMixIn(
    VariableNameSuffixAttrMixIn,
    AnimationFillColorMixIn,
    RevertMixIn,
    AttrLinkingMixIn,
):

    _fill_color: String

    @property
    @add_debug_info_setting(module_name=__name__)
    def fill_color(self) -> String:
        """
        Get this instance's fill color.

        Returns
        -------
        fill_color : String
            Current fill color (hexadecimal string, e.g., '#00aaff').
            If not be set, this interface returns a blank string.

        References
        ----------
        - GraphicsBase fill_color interface
            - https://simon-ritchie.github.io/apysc/en/graphics_base_fill_color.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color="#0af")
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> rectangle.fill_color = ap.String("#f0a")
        >>> rectangle.fill_color
        String('#ff00aa')
        """
        import apysc as ap
        from apysc._type import value_util

        self._initialize_fill_color_if_not_initialized()
        fill_color: ap.String = value_util.get_copy(value=self._fill_color)
        return fill_color

    @fill_color.setter
    @arg_validation_decos.is_hex_color_code_format(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def fill_color(self, value: String) -> None:
        """
        Update this instance's fill color.

        Parameters
        ----------
        value : String
            Fill color to set.

        References
        ----------
        - GraphicsBase fill_color interface
            - https://simon-ritchie.github.io/apysc/en/graphics_base_fill_color.html  # noqa
        """
        self._update_fill_color_and_skip_appending_exp(value=value)
        self._append_fill_color_update_expression()

        self._append_applying_new_attr_val_exp(
            new_attr=self._fill_color, attr_name="fill_color"
        )
        self._append_attr_to_linking_stack(
            attr=self._fill_color, attr_name="fill_color"
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_fill_color_update_expression(self) -> None:
        """
        Append the fill color updating expression.
        """
        import apysc as ap

        expression: str = f'{self.variable_name}.fill("{self.fill_color}");'
        ap.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _set_initial_fill_color_if_not_blank(
        self, *, fill_color: Union[str, String]
    ) -> None:
        """
        Set the initial fill color if a specified value
        is not a blank string.

        Parameters
        ----------
        fill_color : str or String
            Fill color (hexadecimal string, e.g., '#00aaff').
        """
        import apysc as ap

        self._initialize_fill_color_if_not_initialized()
        if fill_color == "":
            return
        if isinstance(fill_color, ap.String):
            fill_color_: ap.String = fill_color
        else:
            suffix: str = self._get_attr_variable_name_suffix(
                attr_identifier="fill_color"
            )
            fill_color_ = String(fill_color, variable_name_suffix=suffix)
        self._update_fill_color_and_skip_appending_exp(value=fill_color_)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _update_fill_color_and_skip_appending_exp(self, *, value: String) -> None:
        """
        Update fill color and skip appending expression.

        Parameters
        ----------
        value : String
            Fill color to set.
        """
        from apysc._color import color_util

        value = color_util.complement_hex_color(hex_color_code=value)
        self._initialize_fill_color_if_not_initialized()
        self._fill_color.value = value

    @final
    @add_debug_info_setting(module_name=__name__)
    def _initialize_fill_color_if_not_initialized(self) -> None:
        """
        Initialize the fill_color attribute if this interface
        does not initialize it yet.
        """
        if hasattr(self, "_fill_color"):
            return
        suffix: str = self._get_attr_variable_name_suffix(attr_identifier="fill_color")
        self._fill_color = String(
            "",
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

    _fill_color_snapshots: Dict[str, str]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_fill_color_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_fill_color_snapshots",
            value=self._fill_color._value,
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._fill_color._value = self._fill_color_snapshots[snapshot_name]



"""This module is for each language enum definition.
"""

from enum import Enum


class Lang(Enum):
    """Translation target languages definitions."""

    EN = "en"
    JP = "jp"


class _UndefinedStrValue(Exception):
    pass


def get_lang_from_str_value(*, str_value: str) -> Lang:
    """
    Get a language enum value from a specified string value.

    Parameters
    ----------
    str_value : str
        A target language string value.

    Returns
    -------
    lang : Lang
        A target language enum value.

    Raises
    ------
    _UndefinedStrValue
        If a specified string value is undefined.
    """
    for lang in Lang:
        if lang.value == str_value:
            return lang
    raise _UndefinedStrValue(
        f"A specified language string value is undefined: {str_value}"
    )



"""Class implementation for the CSS mix-in.
"""

from typing import Dict
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.attr_to_apysc_val_from_builtin_mixin import (
    AttrToApyscValFromBuiltinMixIn,
)
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.string import String
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._validation import arg_validation_decos


class CssMixIn(VariableNameMixIn, RevertMixIn, AttrToApyscValFromBuiltinMixIn):

    _css: Dict[str, String]

    @final
    def _initialize_css_if_not_initialized(self) -> None:
        """
        Initialize the _css attribute if this interface does not
        initialize it yet.
        """
        if hasattr(self, "_css"):
            return
        self._css = {}

    @final
    @arg_validation_decos.is_string(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def get_css(self, *, name: Union[str, String]) -> String:
        """
        Get a CSS value string.

        Parameters
        ----------
        name : str or String
            CSS name (e.g., 'display').

        Returns
        -------
        css : ap.String
            CSS value.

        References
        ----------
        - Display object get_css and set_css interfaces
            - https://simon-ritchie.github.io/apysc/en/display_object_get_and_set_css.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color="#0af", alpha=0.5)
        >>> sprite.set_css(name="display", value="none")
        >>> sprite.get_css(name="display")
        String('none')
        """
        import apysc as ap
        from apysc._converter import to_builtin_val_from_apysc

        self._initialize_css_if_not_initialized()
        name_: str = to_builtin_val_from_apysc.get_builtin_str_from_apysc_val(
            string=name
        )
        if name_ in self._css:
            css: ap.String = self._css[name_]._copy()
        else:
            css = ap.String("")
        self._append_get_css_expresion(name=name, css=css)
        return css

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_get_css_expresion(
        self, *, name: Union[str, String], css: String
    ) -> None:
        """
        Append a CSS getter expression string.

        Parameters
        ----------
        name : str or String
            CSS name (e.g., 'display').
        css : String
            CSS value.
        """
        import apysc as ap
        from apysc._type import value_util

        name_value_str: str = value_util.get_value_str_for_expression(value=name)
        css_value_str: str = value_util.get_value_str_for_expression(value=css)
        expression: str = (
            f"{css_value_str} = {self.variable_name}." f"css({name_value_str});"
        )
        ap.append_js_expression(expression=expression)

    @final
    @arg_validation_decos.is_string(arg_position_index=1)
    @arg_validation_decos.is_string(arg_position_index=2)
    @add_debug_info_setting(module_name=__name__)
    def set_css(self, *, name: Union[str, String], value: Union[str, String]) -> None:
        """
        Set a specified value string to the CSS.

        Parameters
        ----------
        name : str or String
            CSS name (e.g., 'display').
        value : str or String
            A CSS value string (e.g., 'none').

        References
        ----------
        - Display object get_css and set_css interfaces
            - https://simon-ritchie.github.io/apysc/en/display_object_get_and_set_css.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color="#0af", alpha=0.5)
        >>> sprite.set_css(name="display", value="none")
        >>> sprite.get_css(name="display")
        String('none')
        """
        import apysc as ap
        from apysc._converter import to_builtin_val_from_apysc

        self._initialize_css_if_not_initialized()
        name_: str = to_builtin_val_from_apysc.get_builtin_str_from_apysc_val(
            string=name
        )
        value_: ap.String = self._get_copied_string_from_builtin_val(
            string=value, attr_identifier="css"
        )
        self._css[name_] = value_
        self._append_set_css_expression(name=name, value=value)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_set_css_expression(
        self, *, name: Union[str, String], value: Union[str, String]
    ) -> None:
        """
        Append a CSS setter expression string.

        Parameters
        ----------
        name : str or String
            CSS name (e.g., 'display').
        value : str or String
            A CSS value string (e.g., 'none').
        """
        import apysc as ap
        from apysc._type import value_util

        name_value_str: str = value_util.get_value_str_for_expression(value=name)
        value_str: str = value_util.get_value_str_for_expression(value=value)
        expression: str = f"{self.variable_name}.css({name_value_str}, {value_str});"
        ap.append_js_expression(expression=expression)

    _css_snapshot: Dict[str, Dict[str, String]]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make values' snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_css_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_css_snapshot", value={**self._css}, snapshot_name=snapshot_name
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert values if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._css = self._css_snapshot[snapshot_name]
